<?php

function tz_add_meta_box( $meta_box ) {
  if ( ! is_array( $meta_box ) )
    return false;

  $callback = create_function( '$post,$meta_box', 'tz_create_meta_box( $post, $meta_box["args"] );' );

  add_meta_box( $meta_box['id'], $meta_box['title'], $callback, $meta_box['page'], $meta_box['context'], $meta_box['priority'], $meta_box );
}

function tz_create_meta_box( $post, $meta_box ) {
	$wp_version = get_bloginfo( 'version' );

  if ( ! is_array( $meta_box ) )
		return false;

  if ( isset( $meta_box['description'] ) && $meta_box['description'] != '' )
  	echo '<p>' . $meta_box['description'] . '</p>';

	wp_nonce_field( basename( __FILE__ ), 'tz_meta_box_nonce' );

	echo '<table class="form-table tz-metabox-table">';

	foreach( $meta_box['fields'] as $field ) {
		$meta = get_post_meta( $post->ID, $field['id'], true );

		echo '<tr><th><label for="' . $field['id'] . '"><strong>' . $field['name'] . '</strong><span>' . $field['desc'] . '</span></label></th>';

		switch( $field['type'] ) {
			case 'text':
				echo '<td><input type="text" name="tz_meta[' . $field['id'] . ']" id="' . $field['id'] . '" value="' . ( $meta ? $meta : $field['std'] ) . '" size="30" /></td>';
			break;

			case 'textarea':
				echo '<td><textarea name="tz_meta[' . $field['id'] . ']" id="' . $field['id'] . '" rows="8" cols="5">' . ( $meta ? $meta : $field['std'] ) . '</textarea></td>';
			break;

			case 'select':
				echo '<td><select name="tz_meta[' . $field['id'] . ']" id="' . $field['id'] . '">';

				foreach ( $field['options'] as $key => $option ){
					echo '<option value="' . $key . '"';

					if ( $meta ) {
						if ( $meta == $key )
							echo ' selected="selected"';
					} else {
						if ( $field['std'] == $key )
							echo ' selected="selected"';
					}

					echo'>' . $option . '</option>';
				}

				echo'</select></td>';
			break;

			case 'file':
				if ( version_compare( $wp_version, '3.4.2', '>' ) ) {
			?>
				<script>
					jQuery(function($) {
						var frame;

						$('#<?php echo $field['id']; ?>_button').on('click', function(e) {
							e.preventDefault();

							var options = {
								state: 'insert',
								frame: 'post'
							};

							frame = wp.media(options).open();

							frame.menu.get('view').unset('gallery');
							frame.menu.get('view').unset('featured-image');

							frame.toolbar.get('view').set({
								insert: {
									style: 'primary',
									text: 'Insert',

									click: function() {
										var models = frame.state().get('selection'),
											url = models.first().attributes.url;

										$('#<?php echo $field['id']; ?>').val( url );

										frame.close();
									}
								}
							});
						});
					});
				</script>
			<?php
				}

				echo '<td><input type="text" name="tz_meta[' . $field['id'] . ']" id="' . $field['id'] . '" value="' . ( $meta ? $meta : $field['std'] ) . '" size="30" class="file" /> <input type="button" class="button" name="' . $field['id'] . '_button" id="' . $field['id'] . '_button" value="Browse" /></td>';
			break;

			case 'images':
				if ( version_compare( $wp_version, '3.4.2', '>' ) ) {
			?>
				<script>
					jQuery(function($) {
						var frame,
						    images = '<?php echo get_post_meta( $post->ID, '_tz_image_ids', true ); ?>',
						    selection = loadImages(images);

						$('#tz_images_upload').on('click', function(e) {
							e.preventDefault();

							var options = {
								title: '<?php _e( 'Create Featured Gallery', 'schools' ); ?>',
								state: 'gallery-edit',
								frame: 'post',
								selection: selection
							};

							if( frame || selection )
								options['title'] = '<?php _e( 'Edit Featured Gallery', 'schools' ); ?>';

							frame = wp.media(options).open();

							frame.menu.get('view').unset('cancel');
							frame.menu.get('view').unset('separateCancel');
							frame.menu.get('view').get('gallery-edit').el.innerHTML = '<?php _e( 'Edit Featured Gallery', 'schools' ); ?>';
							frame.content.get('view').sidebar.unset('gallery');

							overrideGalleryInsert();
							frame.on( 'toolbar:render:gallery-edit', function() {
								overrideGalleryInsert();
							});

							frame.on( 'content:render:browse', function( browser ) {
								if ( ! browser )
									return;

								browser.sidebar.on('ready', function(){
									browser.sidebar.unset('gallery');
								});

								browser.toolbar.on('ready', function() {
									if ( browser.toolbar.controller._state == 'gallery-library' ) {
										browser.toolbar.$el.hide();
									}
								});
							});

							frame.state().get('library').on( 'remove', function() {
								var models = frame.state().get('library');

								if ( models.length == 0 ) {
									selection = false;
									$.post(ajaxurl, { ids: '', action: 'tz_save_images', post_id: tz_ajax.post_id, nonce: tz_ajax.nonce });
								}
							});

							function overrideGalleryInsert() {
								frame.toolbar.get('view').set({
									insert: {
										style: 'primary',
										text: '<?php _e( 'Save Featured Gallery', 'schools' ); ?>';

										click: function() {
											var models = frame.state().get('library'),
												  ids = '';

											models.each( function( attachment ) {
												ids += attachment.id + ','
											});

											this.el.innerHTML = '<?php _e( 'Saving ...', 'schools' ); ?>';

											$.ajax({
												type: 'POST',
												url: ajaxurl,
												data: {
													ids: ids,
													action: 'tz_save_images',
													post_id: tz_ajax.post_id,
													nonce: tz_ajax.nonce
												},
												success: function(){
													selection = loadImages(ids);
													$('#_tz_image_ids').val( ids );
													frame.close();
												},
												dataType: 'html'
											}).done( function( data ) {
												$('.tz-gallery-thumbs').html( data );
											});
										}
									}
								});
							}
						});

						function loadImages(images) {
							if( images ){
								var shortcode = new wp.shortcode({
									tag:    'gallery',
									attrs:   { ids: images },
									type:   'single'
								});

								var attachments = wp.media.gallery.attachments( shortcode );

								var selection = new wp.media.model.Selection( attachments.models, {
									props:    attachments.props.toJSON(),
									multiple: true
								});

								selection.gallery = attachments.gallery;
								selection.more().done( function() {
									selection.props.set({ query: false });
									selection.unmirror();
									selection.props.unset('orderby');
								});

								return selection;
							}

							return false;
						}
					});
				</script>
			<?php
				$meta = get_post_meta( $post->ID, '_tz_image_ids', true );
				$thumbs_output = '';
				$button_text = ( $meta ) ? __( 'Edit Gallery', 'schools' ) : $field['std'];

				if ( $meta ) {
					$field['std'] = __( 'Edit Gallery', 'schools' );
					$thumbs = explode( ',', $meta );
					$thumbs_output = '';
					foreach ( $thumbs as $thumb )
						$thumbs_output .= '<li>' . wp_get_attachment_image( $thumb, array( 32,32 ) ) . '</li>';
				}

			    echo
			    	'<td>
			    		<input type="button" class="button" name="' . $field['id'] . '" id="tz_images_upload" value="' . $button_text . '" />
			    		<input type="hidden" name="tz_meta[_tz_image_ids]" id="_tz_image_ids" value="' . ( $meta ? $meta : 'false' ) . '" />
						<ul class="tz-gallery-thumbs">' . $thumbs_output . '</ul>
			    	</td>';
			    } else {
			    	echo '<td><input type="button" class="button" name="' . $field['id'] . '" id="tz_images_upload" value="' . $field['std'] . '" /></td>';
			    }
			break;

			case 'radio':
				echo '<td>';

				foreach ( $field['options'] as $key => $option ) {
					echo '<label class="radio-label"><input type="radio" name="tz_meta[' . $field['id'] . ']" value="' . $key . '" class="radio"';

					if ( $meta ) {
						if ( $meta == $key )
							echo ' checked="checked"';
					} else {
						if ( $field['std'] == $key )
							echo ' checked="checked"';
					}

					echo ' /> ' . $option . '</label> ';
				}

				echo '</td>';
			break;

			case 'checkbox':
			    echo '<td>';

			    $val = '';
          if ( $meta ) {
            if ( $meta == 'on' )
					    $val = ' checked="checked"';
          } else {
            if ( $field['std'] == 'on' )
						  $val = ' checked="checked"';
          }

          echo '<input type="hidden" name="tz_meta[' . $field['id'] . ']" value="off" /><input type="checkbox" id="' . $field['id'] . '" name="tz_meta[' . $field['id'] . ']" value="on"' . $val . ' /> ';
			    echo '</td>';
			break;
		}

		echo '</tr>';
	}

	echo '</table>';
}

add_action( 'save_post', 'tz_save_meta_box' );
function tz_save_meta_box( $post_id ) {
	if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE )
		return;

	if ( ! isset( $_POST['tz_meta'] ) || ! isset( $_POST['tz_meta_box_nonce'] ) || ! wp_verify_nonce( $_POST['tz_meta_box_nonce'], basename( __FILE__ ) ) )
		return;

	if ( 'page' == $_POST['post_type'] ) {
		if ( ! current_user_can( 'edit_page', $post_id ) )
      return;
	} else {
		if ( ! current_user_can( 'edit_post', $post_id ) )
      return;
	}

	foreach( $_POST['tz_meta'] as $key=>$val )
		update_post_meta( $post_id, $key, stripslashes( htmlspecialchars( $val ) ) );
}

add_action( 'wp_ajax_tz_save_images', 'tz_save_images' );
function tz_save_images() {
	if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE )
		return;

	if ( ! isset( $_POST['ids'] ) || ! isset( $_POST['nonce'] ) || ! wp_verify_nonce( $_POST['nonce'], 'tz-ajax' ) )
		return;

	if ( ! current_user_can( 'edit_posts' ) )
		return;

	$ids = strip_tags( rtrim( $_POST['ids'], ',' ) );
	update_post_meta( $_POST['post_id'], '_tz_image_ids', $ids );

	$thumbs = explode( ',', $ids );
	$thumbs_output = '';
	foreach( $thumbs as $thumb ) {
		$thumbs_output .= '<li>' . wp_get_attachment_image( $thumb, array( 32,32 ) ) . '</li>';
	}

	echo $thumbs_output;

	die();
}

add_action( 'admin_print_styles', 'tz_metabox_style' );
function tz_metabox_style() {
	global $post;

	$wp_version = get_bloginfo('version');

	wp_enqueue_style( 'tz-metabox-css', get_template_directory_uri() . '/functions/styles/metaboxes.css');

	wp_enqueue_script('media-upload');
	if ( version_compare( $wp_version, '3.4.2', '<=' ) ) {
		wp_enqueue_script( 'thickbox' );
		wp_register_script( 'tz-upload', get_template_directory_uri() . '/functions/scripts/upload.js', array( 'jquery','media-upload','thickbox' ) );
		wp_enqueue_script( 'tz-upload' );

		wp_enqueue_style('thickbox');
	}

	if( isset( $post ) ) {
		wp_localize_script( 'jquery', 'tz_ajax', array(
		    'post_id' => $post->ID,
		    'nonce' => wp_create_nonce( 'tz-ajax' )
		) );
	}
}

?>
